@@ -32,8 +32,10 @@ details.  */
 #include <java/security/ProtectionDomain.h>
 #include <java/lang/ClassFormatError.h>
 #include <java/lang/StringBuffer.h>
+#include <java/lang/SecurityManager.h>
 #include <java/lang/Runtime.h>
 #include <java/util/HashSet.h>
+#include <java/lang/SecurityException.h>
 #include <java/lang/VirtualMachineError.h>
 
 java::lang::Class *
@@ -47,6 +49,9 @@ java::lang::VMClassLoader::defineClass (java::lang::ClassLoader *loader,
   jclass klass = VMCompiler::compileClass(loader, name, data,
 					  offset, length, pd);
 
+  if (klass)
+    _Jv_RegisterInitiatingLoader (klass, klass->loader);
+
 #ifdef INTERPRETER
   if (klass == NULL)
     {
@@ -91,7 +96,7 @@ java::lang::VMClassLoader::defineClass (java::lang::ClassLoader *loader,
 	}
 
       // if everything proceeded sucessfully, we're loaded.
-      JvAssert (klass->state == JV_STATE_LOADED);
+      JvAssert (klass->state == JV_STATE_READ);
     }
 #endif // INTERPRETER
 
@@ -201,6 +206,16 @@ java::lang::VMClassLoader::nativeFindClass (jstring name)
 jclass
 java::lang::VMClassLoader::loadClass(jstring name, jboolean resolve)
 {
+  using namespace ::java::lang;
+
+  SecurityManager *sm = (SecurityManager *)SecurityManager::current;
+  if (sm)
+    {
+      jint lastDot = name->lastIndexOf('.');
+      if (lastDot != -1)
+	sm->checkPackageAccess(name->substring(0, lastDot));
+    }
+
   // We try the boot loader first, so that the endorsed directory
   // overrides compiled-in classes.
   jclass klass = NULL;
